import * as nt from 'noodl-types';
import { Draft } from 'immer';
import log from '../utils/log';
import IndexRepository from '../db/IndexRepository';
import * as t from '../types';
export default class CADL {
    #private;
    cadlEndpoint: nt.AppConfig | null;
    cadlVersion: nt.Env;
    initCallQueue: any[];
    myBaseUrl: string;
    verificationRequest: {
        timer: number;
        phoneNumber: string;
    };
    constructor({ configUrl, cadlVersion, aspectRatio, dbConfig, loglevel, SearchClient, }: {
        configUrl: string;
        cadlVersion: nt.Env;
        aspectRatio?: number;
        loglevel?: keyof typeof log.levels;
        dbConfig?: any;
        SearchClient?: InstanceType<any>;
    });
    get dispatch(): t.Dispatch<t.DispatchPayload>;
    get emit(): <P extends Record<string, any> = Record<string, any>>({ type, payload }: {
        type: "SET_CACHE" | "ADD_BUILTIN_FNS" | "DELETE_PAGE" | "EDIT_DRAFT" | "SET_ROOT_PROPERTIES" | "SET_LOCAL_PROPERTIES" | "SET_VALUE";
        payload: P;
    }) => Record<string, any>;
    get indexRepository(): IndexRepository;
    /**
     * Loads noodl config if not already loaded
     * Sets CADL version, baseUrl, assetsUrl, and root
     */
    init({ onFetchPreload, use, }?: {
        onFetchPreload?: (name: string, config: nt.RootConfig, cadlEndpoint: nt.AppConfig) => void;
        use?: {
            BaseDataModel?: Record<string, any>;
            BaseCSS?: Record<string, any>;
            BasePage?: Record<string, any>;
            config?: nt.RootConfig;
            cadlEndpoint?: nt.AppConfig;
        } & {
            [pageName: string]: Record<string, any>;
        };
    }): Promise<void>;
    /**
     *
     * @param pageName
     * @param skip Denotes the keys to skip in the population process
     * @param options Object that takes in set of options for the page
     *
     * @throws {UnableToRetrieveYAML} -When unable to retrieve noodlYAML
     * @throws {UnableToParseYAML} -When unable to parse yaml file
     * @throws {UnableToExecuteFn} -When something goes wrong while executing any init function
     *
     * - initiates cadlObject for page specified
     */
    initPage(pageArg: string | {
        pageName: string;
        cadlYAML: string;
        cadlObject: Record<string, nt.PageObject>;
    }, skip?: string[], options?: Pick<Parameters<CADL['runInit']>[0], 'onBeforeInit' | 'onInit' | 'onAfterInit'> & {
        reload?: boolean;
        builtIn?: Record<string, any>;
        onReceive?(obj: {
            [pageName: string]: any;
        }): Promise<void> | void;
        onAbort?(obj: {
            [pageName: string]: any;
        }): Promise<void> | void;
        onFirstProcess?(obj: {
            [pageName: string]: any;
        }): Promise<void> | void;
        onSecondProcess?(obj: {
            [pageName: string]: any;
        }): Promise<void> | void;
        onFetchPage?: (pageName: string, config: nt.RootConfig, cadlEndpoint: nt.AppConfig) => void;
        shouldAttachRef?: t.ShouldAttachRefFn;
        wrapEvalObjects?: boolean;
    }): Promise<void | {
        aborted: true;
    }>;
    /**
     * @param pageName
     * @returns { object }
     * @throws {UnableToRetrieveYAML} -When unable to retrieve cadlYAML
     * @throws {UnableToParseYAML} -When unable to parse yaml file
     */
    getPage(pageName: string): Promise<[Record<string, nt.PageObject>, string]>;
    /**
     * Used to populate the references of the noodl files.
     *
     * @param ProcessPopulateArgs
     * @param ProcessPopulateArgs.source  The item being de-referenced.
     * @param ProcessPopulateArgs.lookFor  Reference tokens to look for e.g ['.','..'].
     * @param ProcessPopulateArgs.pageName
     * @param ProcessPopulateArgs.skip Keys that should not be de-referenced e.g ['name','country'].
     * @param ProcessPopulateArgs.withFns Choose to attach ecos functions to the source
     *
     * @returns The processed/de-referenced object.
     *
     */
    processPopulate({ source, lookFor, skip, pageName, shouldAttachRef, withFns, wrapEvalObjects, }: {
        source: Record<string, any>;
        lookFor: string[];
        pageName?: string;
        skip?: string[];
        shouldAttachRef?: t.ShouldAttachRefFn;
        withFns?: boolean;
        wrapEvalObjects?: boolean;
    }): Record<string, any>;
    /**
     *
     * @param HandleEvalStringArgs.stringArg The item being de-referenced
     * @param HandleEvalStringArgs.pageName
     *
     * Used as a helper function for emitCall --> evalObject
     */
    handleEvalString({ stringArg, pageName }: {
        stringArg: any;
        pageName: any;
    }): Promise<any>;
    /**
     *
     * @example
     * ```js
     * await handleEvalObject({ object: { '.path@': 5, '.path2@': 6 }, pageName: 'SignIn })
     * ```
     * @param HandleEvalObject.object series of commands in object {key:val} format
     * @param HandleEvalObject.pageName
     *
     * Used as a helper function for emitCall --> evalObject
     */
    handleEvalObject({ object, pageName, }: {
        object: nt.EvalActionObject['object'];
        pageName?: string;
    }): Promise<any>;
    /**
     * @param HandleEvalArray.pageName
     *
     * Used as a helper function for emitCall --> evalObject
     *
     * @example
     * ```
     * const result = await handleEvalArray([
     * 	  { '.path@': 3 },
     *    { 'path2@': 5 },
     *    { if: ['.condition', ifTrue, ifFalse] }
     * ])
     * ```
     */
    handleEvalArray({ array, pageName, }: {
        array: any[];
        pageName?: string;
    }): Promise<any[]>;
    /**
     * @param HandleEvalCommandsArgs.commands Series of commands to evaluate
     * @param HandleEvalCommandsArgs.key Key to a specified command within commands
     *
     * Used as a helper function for emitCall --> evalObject
     */
    handleEvalCommands({ commands, key, pageName }: {
        commands: any;
        key: any;
        pageName: any;
    }): Promise<any>;
    /**
     *
     * @param HandleEvalAssignmentExpressionsArgs.command Assigment command of shape {'.path@':4}
     * @param HandleEvalAssignmentExpressionsArgs.pageName
     *
     * Used as a helper function for emitCall --> evalObject -->  handleEvalCommands
     */
    handleEvalAssignmentExpressions({ pageName, command, key }: {
        pageName: any;
        command: any;
        key: any;
    }): Promise<void>;
    /**
     *
     * @param HandleEvalFunctionArgs.key
     * @param HandleEvalFunctionArgs.pageName
     * @param HandleEvalFunctionArgs.command
     * Used as a helper function for emitCall --> evalObject -->  handleEvalCommands
     */
    handleEvalFunction({ key, pageName, command, }: {
        command: Record<string, any>;
        key: string;
        pageName?: string;
    }): Promise<any>;
    /**
     * Evaluates if objects of shape
     */
    handleIfCommand({ pageName, ifCommand, }: {
        pageName: string;
        ifCommand: nt.IfObject;
    }): Promise<any>;
    /**
     * Used for the actionType 'updateObject'. It updates the value of an object at the given path.
     *
     * @param UpdateObjectArgs
     * @param UpdateObjectArgs.dataKey The path to the property being changed.
     * @param UpdateObjectArgs.dataObject The object that will be updated.
     * @param UpdateObjectArgs.dataObjectKey The specific key of the dataObject to be used as the new value.
     * @emits CADL#stateChanged
     *
     */
    updateObject({ dataKey, dataObject, dataObjectKey, }: t.UpdateObjectArgs): Promise<void>;
    /**
     * Runs the init functions of the page matching the pageName.
     *
     * @param pageObject
     * @param onBeforeInit
     * @param onInit
     * @param onAfterInit
     */
    runInit<Init extends any[]>({ pageObject, onBeforeInit, onInit, onAfterInit, }: {
        pageObject: Record<string, any>;
        onBeforeInit?(init: Init): Promise<void> | void;
        onInit?(current: any, index: number, init: Init): Promise<void> | void;
        onAfterInit?(error: null | Error, init: Init): Promise<void> | void;
    }): Promise<Record<string, any>>;
    /**
     * Sets either the user or meetroom value from localStorage to the corresponding root value in memory
     *
     * @param key "user" | "meetroom"
     *
     */
    setFromLocalStorage(key: 'user' | 'meetroom'): void;
    /**
     * Used to mutate the draft state.
     *
     * @param callback Function used to update the state
     */
    editDraft(callback: (draft: Draft<t.Root>) => void): void;
    /**
     * Used to handle the emit syntax, where a series
     * of actions can be called given a common variable(s)
     * @param dataKey -object with variables e.g {var1:{name:'tom'}}
     * @param actions -an array of commands/actions to be performed
     * @param pageName
     * @returns { any[] }
     */
    emitCall({ dataKey, actions, pageName, }: t.EmitCallArgs): Promise<any[]>;
    getApiCache(cacheIndex: any): any;
    getState(): {
        ids: Record<string, {
            type: t.ActiveType;
        }>;
        queue: t.ActiveQueueObject<string>[];
        history: any[];
        init: {
            initiating: boolean;
            done: boolean;
            error: Error | null;
        };
    };
    getSubscribers(): t.ActiveQueueSubscribers;
    on(evt: t.ActiveQueueSubscribeEvent, callback: t.ActiveQueueSubscriber): undefined;
    get baseUrl(): string;
    set baseUrl(baseUrl: string);
    get cadlBaseUrl(): string | undefined;
    set cadlBaseUrl(cadlBaseUrl: string | undefined);
    get assetsUrl(): string;
    set assetsUrl(assetsUrl: string);
    get designSuffix(): any;
    set designSuffix(designSuffix: any);
    get aspectRatio(): number;
    set aspectRatio(aspectRatio: number);
    get root(): t.Root;
    set root(root: t.Root);
    set apiVersion(apiVersion: string);
    get apiVersion(): string;
    get config(): nt.RootConfig | null;
}
